{% load i18n %}
{% trans "Collection privacy" as title_str %}
{% include "wagtailadmin/shared/header.html" with title=title_str icon="no-view" %}

<div class="nice-padding">
    <p>{% trans "This collection has been made private by a parent collection." %}</p>
    <p>{% trans "You can edit the privacy settings on:" %} <a href="{% url 'wagtailadmin_collections:edit' collection_with_restriction.id %}">{{ collection_with_restriction.name }}</a>
</div>
