from __future__ import absolute_import, unicode_literals

from django.apps import AppConfig


class WagtailFormsAppConfig(AppConfig):
    name = 'wagtail.wagtailforms'
    label = 'wagtailforms'
    verbose_name = "Wagtail forms"
