{% load wagtailimages_tags %}
{% load i18n %}
{% trans "Choose a format" as choose_str %}
{% include "wagtailadmin/shared/header.html" with title=choose_str %}

<div class="row row-flush nice-padding">
    <div class="col6">
        {% image image max-800x600 class="show-transparency" %}
    </div>
    <div class="col6">
        <form action="{% url 'wagtailimages:chooser_select_format' image.id %}" method="POST" novalidate>
            {% csrf_token %}
            <ul class="fields">
                {% for field in form %}
                    {% include "wagtailadmin/shared/field_as_li.html" with field=field %}
                {% endfor %}
                <li><input type="submit" value="{% trans 'Insert image' %}"  class="button" /></li>
            </ul>
        </form>
    </div>
</div>
