{% extends "wagtailadmin/shared/field.html" %}
{% load i18n %}
{% trans "Date" %}
{% block form_field %}
   <div class="chooser searchterm-chooser">
       {{ field }}

       <button type="button" id="{{ field.auto_id }}-chooser" class="button button-secondary searchterms-chooser">{% trans "Choose from popular search terms" %}</button>
    </div>
{% endblock %}
