.page404__bg{position:fixed;top:0;left:0;width:100vw;height:100vh;background-color:#358c8b;font-family:Open Sans,Arial,sans-serif;color:#fff}.page404__wrapper{position:absolute;top:50%;left:50%;-ms-transform:translate(-50%,-50%);transform:translate(-50%,-50%);display:-ms-flexbox;display:flex;-ms-flex-pack:center;justify-content:center;-ms-flex-align:center;align-items:center}.wagtail-logo-container__desktop{float:left;width:400px;height:500px}.page404__text-container{float:right;width:600px;height:500px;text-align:center}.page404__header{font-size:6.8em;margin-bottom:.2em;color:inherit}.page404__text{font-size:2.25em;line-height:1.25em;color:inherit}a.button.page404__button{font-size:1.5em;line-height:2em;height:2.5em;padding:0 .5em;background-color:#358c8b;border:4px solid #43b1b0;color:inherit}a.button.page404__button:hover{background-color:#43b1b0}@media screen and (max-width:56.25em){.wagtail-logo-container__desktop{display:none}}@media screen and (max-width:50em){.page404__text-container{width:400px}.page404__header{font-size:5em}}