{% extends "tests/base.html" %}
{% load wagtailcore_tags wagtailimages_tags %}

{% block html_title %}Event: {{ self.title }}{% endblock %}
{% block content %}
  <h2>Event</h2>
  {% if self.feed_image %}
      {% image self.feed_image width-200 class="feed-image" %}
  {% endif %}

  {% with self.categories.all as categories %}
    {% if categories %}
      <ul>
        {% for category in categories %}
          <li>{{ category.name }}</li>
        {% endfor %}
      </ul>
    {% endif %}
  {% endwith %}

  {{ self.body|richtext }}
  <p><a href="{% slugurl 'events' %}">Back to events index</a></p>
{% endblock %}
