section{border-top:1px solid #d9d9d9;margin-top:2em;padding:0 0 2em}section>h2:first-child{margin:0;font-size:1em;background:#e6e6e6;padding:1em;margin-bottom:1em}.palette:after,.palette:before{content:" ";display:table}.palette:after{clear:both}.palette ul{margin-top:0;margin-bottom:0;padding-left:0;list-style-type:none;font-style:normal}.palette ul:after,.palette ul:before{content:" ";display:table}.palette ul:after{clear:both}.palette ul li{list-style-type:none;font-style:normal}.palette li{float:left;width:100px;height:100px;padding:10px;color:#000}.palette .color-teal{background-color:#43b1b0}.palette .color-teal-darker{background-color:#358c8b}.palette .color-teal-dark{background-color:#246060}.palette .color-red{background-color:#cd3238}.palette .color-orange{background-color:#e9b04d}.palette .color-green{background-color:#189370}.palette .color-blue{background-color:#71b2d4}.palette .color-grey-1{background-color:#333}.palette .color-grey-1-1{background-color:#404040}.palette .color-grey-2{background-color:#666}.palette .color-grey-3{background-color:#d9d9d9}.palette .color-grey-4{background-color:#e6e6e6}.palette .color-grey-5{background-color:#fafafa}.palette .color-salmon{background-color:#f37e77}.palette .color-salmon-light{background-color:#fcf2f2}.icons :after,.icons :before{font-size:2em}.icons ul{-moz-column-count:3;column-count:3}.icons li{margin-bottom:1em}.icons .spinner{position:relative}.timepicker{height:150px}