{% load i18n wagtailadmin_tags %}

<h2>{% trans "Explorer" %}</h2>
{% include "wagtailadmin/shared/chooser_breadcrumb.html" with page=parent_page %}

{% if pages %}
    {% include "wagtailadmin/pages/listing/_list_choose.html" with allow_navigation=1 orderable=0 pages=pages parent_page=parent_page %}

    {% url 'wagtailadmin_choose_page_child' parent_page.id as pagination_base_url %}
    {% paginate pages base_url=pagination_base_url classnames="navigate-pages" %}
{% endif %}
