{% extends "wagtailadmin/admin_base.html" %}
{% load staticfiles i18n %}
{% block titletag %}{% trans "Reset password" %}{% endblock %}
{% block bodyclass %}login{% endblock %}

{% block extra_css %}
    {{ block.super }}

    <link rel="stylesheet" href="{% static 'wagtailadmin/css/layouts/login.css' %}" type="text/css" />
{% endblock %}

{% block furniture %}
    <div class="content-wrapper">
        <h1>{% trans "Check your email" %}</h1>
        <p>{% trans "A link to reset your password has been emailed to you." %}</p>
    </div>
{% endblock %}
