from __future__ import absolute_import, unicode_literals

from django.apps import AppConfig


class WagtailUsersAppConfig(AppConfig):
    name = 'wagtail.wagtailusers'
    label = 'wagtailusers'
    verbose_name = "Wagtail users"
