from __future__ import absolute_import, unicode_literals

from django.apps import AppConfig


class WagtailStyleGuideAppConfig(AppConfig):
    name = 'wagtail.contrib.wagtailstyleguide'
    label = 'wagtailstyleguide'
    verbose_name = "Wagtail style guide"
