from __future__ import absolute_import, unicode_literals

from django.apps import AppConfig


class WagtailSnippetsAppConfig(AppConfig):
    name = 'wagtail.wagtailsnippets'
    label = 'wagtailsnippets'
    verbose_name = "Wagtail snippets"
