{% extends "wagtailadmin/admin_base.html" %}
{% load wagtailadmin_tags wagtailcore_tags staticfiles i18n %}

{% block furniture %}
    <div class="nav-wrapper">
        <div class="inner">
            <a href="{% url 'wagtailadmin_home' %}" class="logo" title="Wagtail v.{% wagtail_version %}">
                {% block branding_logo %}
                    {# Mobile-only logo: #}
                    <div class="wagtail-logo-container__mobile u-hidden@medium">
                        <img class="wagtail-logo wagtail-logo__full" src="{% static 'wagtailadmin/images/wagtail-logo.svg' %}" alt="Wagtail" width="80" />
                    </div>

                    {# Desktop logo (animated): #}
                    {% include "wagtailadmin/shared/animated_logo.html" %}
                {% endblock %}
                <span class="u-hidden@medium">{% trans "Dashboard" %}</span>
            </a>

            {% menu_search %}
            {% main_nav %}

        </div>
        <div class="explorer__wrapper" data-explorer-menu></div>
    </div>

    <div class="content-wrapper">
        <div class="content">
            {# Always show messages div so it can be appended to by JS #}
            <div class="messages">
                {% if messages %}
                    <ul>
                        {% for message in messages %}
                            <li class="{% message_tags message %}">{{ message|safe }}</li>
                        {% endfor %}
                    </ul>
                {% endif %}
            </div>

            <div id="nav-toggle" class="nav-toggle icon text-replace">{% trans "Menu" %}</div>

            {% block content %}{% endblock %}
        </div>
    </div>
{% endblock %}
