{% load i18n %}
{% if form_pages %}
    {% include "wagtailforms/list_forms.html" %}

    {% include "wagtailadmin/shared/pagination_nav.html" with items=form_pages linkurl="wagtailforms:index" %}
{% else %}
    <p>{% trans "No form pages have been created." %}</p>
{% endif %}
